# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#
# Description
# ==============================================================================
#
#    Tests of tables with UTF-8 characters.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

@testset "Strings with characters that have variable width" begin
    matrix = ["😄"^10 "😄"^10 "😅"^10; "🧐"^5 "🥺"^5 "😇"^5; "a"^10 "a"^10 "a"^10]
    header = (["😋"^5,  "😁"^10, "🤣"^15], ["⚡️"^15, "👽"^10, "🤩"^5])

    expected = """
┌────────────────────────────────┬──────────────────────┬────────────────────────────────┐
│ 😋😋😋😋😋                     │ 😁😁😁😁😁😁😁😁😁😁 │ 🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣 │
│ ⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️ │ 👽👽👽👽👽👽👽👽👽👽 │                     🤩🤩🤩🤩🤩 │
├────────────────────────────────┼──────────────────────┼────────────────────────────────┤
│ 😄😄😄😄😄😄😄😄😄😄           │ 😄😄😄😄😄😄😄😄😄😄 │           😅😅😅😅😅😅😅😅😅😅 │
│ 🧐🧐🧐🧐🧐                     │      🥺🥺🥺🥺🥺      │                     😇😇😇😇😇 │
│ aaaaaaaaaa                     │      aaaaaaaaaa      │                     aaaaaaaaaa │
└────────────────────────────────┴──────────────────────┴────────────────────────────────┘
"""

    result = pretty_table(
        String,
        matrix;
        header = header,
        alignment = [:l, :c, :r]
    )

    @test result == expected

    # Crop
    # ----

    expected = """
┌────────────────────────────────┬──────────────────────┬─────────────────────────────────
│ 😋😋😋😋😋                     │ 😁😁😁😁😁😁😁😁😁😁 │ 🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣 ⋯
│ ⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️ │ 👽👽👽👽👽👽👽👽👽👽 │                     🤩🤩🤩🤩🤩 ⋯
├────────────────────────────────┼──────────────────────┼─────────────────────────────────
│ 😄😄😄😄😄😄😄😄😄😄           │ 😄😄😄😄😄😄😄😄😄😄 │           😅😅😅😅😅😅😅😅😅😅 ⋯
│ 🧐🧐🧐🧐🧐                     │      🥺🥺🥺🥺🥺      │                     😇😇😇😇😇 ⋯
│ aaaaaaaaaa                     │      aaaaaaaaaa      │                     aaaaaaaaaa ⋯
└────────────────────────────────┴──────────────────────┴─────────────────────────────────
"""

    result = pretty_table(
        String,
        matrix;
        header = header,
        alignment = [:l, :c, :r],
        crop = :both,
        display_size = (0, 90)
    )

    @test result == expected

    expected = """
┌────────────────────────────────┬──────────────────────┬────────────────────────────────
│ 😋😋😋😋😋                     │ 😁😁😁😁😁😁😁😁😁😁 │ 🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣🤣  ⋯
│ ⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️⚡️ │ 👽👽👽👽👽👽👽👽👽👽 │                     🤩🤩🤩🤩  ⋯
├────────────────────────────────┼──────────────────────┼────────────────────────────────
│ 😄😄😄😄😄😄😄😄😄😄           │ 😄😄😄😄😄😄😄😄😄😄 │           😅😅😅😅😅😅😅😅😅  ⋯
│ 🧐🧐🧐🧐🧐                     │      🥺🥺🥺🥺🥺      │                     😇😇😇😇  ⋯
│ aaaaaaaaaa                     │      aaaaaaaaaa      │                     aaaaaaaaa ⋯
└────────────────────────────────┴──────────────────────┴────────────────────────────────
                                                                         1 column omitted
"""

    result = pretty_table(
        String,
        matrix;
        header = header,
        alignment = [:l, :c, :r],
        crop = :both,
        display_size = (0, 89)
    )

    @test result == expected
end
